from typing import Dict, Any, List

from data_watch.api.model import SearchRequest
from data_watch.api.service.job.__util import (
    _get_job_request_info,
    _get_jobs_request_info,
    _create_job_request_info,
    _update_job_request_info,
    _delete_job_request_info,
    _search_jobs_request_info,
    _get_job_tags_request_info,
)
from data_watch.api.service.job.model import JobRequest
from data_watch.common.decorators import data_request

from data_watch.common.requests import (
    __get_request,
    __post_request,
    __put_request,
    __delete_request,
)


@data_request
def get_job(id: str) -> Dict[str, Any]:
    """
    Get job by ID.

    Args:
        id (str): Job ID

    Returns:
        Dict[str, Any]: Job response
    """
    request_info = _get_job_request_info(id)
    return __get_request(**request_info)


@data_request
def get_jobs() -> List[Dict[str, Any]]:
    """
    Get all jobs.

    Returns:
        List[Dict[str, Any]]: List of job response
    """
    request_info = _get_jobs_request_info()
    return __get_request(**request_info)


@data_request
def search_jobs(request: SearchRequest) -> List[Dict[str, Any]]:
    """
    Search all Jobs.

    Parameters:
        request (SearchRequest): A request carrying the search query

    Returns:
        List[Dict[str, Any]]: List of job responses
    """
    request_info = _search_jobs_request_info(request)
    return __post_request(**request_info)


@data_request
def create_job(request: JobRequest) -> Dict[str, Any]:
    """
    Create job.

    Args:
        request (JobRequest): Job request

    Returns:
        Dict[str, Any]: Job response
    """
    request_info = _create_job_request_info(request)
    return __put_request(**request_info)


@data_request
def update_job(id: str, request: JobRequest) -> Dict[str, Any]:
    """
    Update job.

    Args:
        id (str): Job ID
        request (JobRequest): Job request

    Returns:
        Dict[str, Any]: Job response
    """
    request_info = _update_job_request_info(id, request)
    return __post_request(**request_info)


@data_request
def delete_job(id: str) -> None:
    """
    Delete job by ID.

    Args:
        id (str): Job ID

    Returns:
        None
    """
    request_info = _delete_job_request_info(id)
    return __delete_request(**request_info)


@data_request
def get_job_tags() -> List[str]:
    """
    Get all the tags used by jobs

    Returns:
        List[str]: A list of all the tags used for jobs
    """
    request_info = _get_job_tags_request_info()
    return __get_request(**request_info)
